import { z } from "https://deno.land/x/zod@v3.22.4/mod.ts";
import { log_from } from "./utils.ts";
const log = (...args: unknown[]) => log_from(import.meta.url, ...args);

/**
 * Wraps the CRI API.
 */
export class CriApi {
	/**
	 * note : a CRI token is actually
	 */
	private token;

	public constructor(token: string) {
		this.token = token;
	}

	/**
	 * Fetches an array of the groups an user has been part of.
	 */
	public async groups_of(login: string) {
		log(`Fetching groups of '${login}'.`);
		const response = await fetch_json_auth(`https://cri.epita.fr/api/v2/users/${login}/`, this.token);
		const group = z.object({ slug: z.string() });
		const parser = z.object({
			primary_group: group,
			groups_history: z.array(z.object({
				group,
				graduation_year: z.number().or(z.null()),
			})),
			current_groups: z.array(group),
		});
		const parsed = parser.parse(response);
		const result = new Set<string>();
		result.add(parsed.primary_group.slug);
		for (const group of parsed.current_groups) result.add(group.slug);
		for (const { group } of parsed.groups_history) result.add(group.slug);
		return Array.from(result.values());
	}

	/**
	 * Tests wether a given login exists within the CRI registry.
	 */
	public async user_exists(login: string) {
		log(`Testing existence of user '${login}'.`);
		const response = await fetch(`https://cri.epita.fr/api/v2/users/${login}/`, {
			headers: {
				accept: "application/json",
				authorization: "Basic " + this.token,
			},
		});
		const found = response.status === 200;
		return found;
	}

	public async get_all_groups() {
		const result = [] as string[];
		const response_parser = z.object({
			next: z.string().or(z.null()),
			results: z.array(z.object({
				slug: z.string(),
			})),
		});
		let next_url = `https://cri.epita.fr/api/v2/groups/`;
		while (true) {
			const response = await fetch_json_auth(next_url, this.token);
			const parsed = response_parser.parse(response);
			result.push(...parsed.results.map((g) => g.slug));
			if (parsed.next === null) break;
			next_url = parsed.next;
		}
		return result;
	}
}

async function fetch_json_auth(url: string, token: string) {
	const response = await fetch(url, {
		headers: {
			accept: "application/json",
			authorization: "Basic " + token,
		},
	});
	return await response.json() as unknown;
}
